<?php

/**
 * E-goi.com
 *
 * @title      E-Goi Multi-channel Marketing
 * @package    E-Goi
 * @copyright  Copyright (c) 2012-2023 E-Goi - https://e-goi.com
 */

namespace Egoi\Marketing\Controller\Adminhtml\Account;

use Egoi\Marketing\Helper\Data;
use Magento\Backend\App\Action;

/**
 * Class Transactional
 *
 * @package Egoi\Marketing\Controller\Adminhtml\Account
 */
class Transactional extends \Magento\Backend\App\Action
{

    /**
     * @var \Magento\Framework\App\Config\ScopeConfigInterface
     */
    protected $_scopeConfig;

    /**
     * @var Data
     */
    protected $helper;

    /**
     * Transactional constructor.
     *
     * @param Data                                               $helper
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
     * @param Action\Context                                     $context
     */
    public function __construct(
        \Egoi\Marketing\Helper\Data                        $helper,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        Action\Context                                     $context
    )
    {

        parent::__construct($context);

        $this->helper = $helper;
        $this->_scopeConfig = $scopeConfig;
    }

    /**
     *
     */
    public function execute()
    {

        try {

            $transport = $this->helper->getSmtpTransport();

            $salesSender = $this->_scopeConfig->getValue('trans_email/ident_general/name');
            $salesEmail = $this->_scopeConfig->getValue('trans_email/ident_general/email');

            if ($this->helper->versionCompare('2.2.8')) {

                $message = new \Laminas\Mail\Message;
                $message->setBody(
                    'If you are receiving this message, everything seems to be ok with your SMTP configuration'
                );
                $message->setFrom($salesEmail, $salesSender)
                        ->addTo($salesEmail, $salesSender)
                        ->setSubject('E-Goi / Magento - Test');

                $transport = new \Laminas\Mail\Transport\Smtp();
                $options = new \Laminas\Mail\Transport\SmtpOptions(
                    [
                        'name'              => 'localhost',
                        'host'              => $this->helper->getSmtpServer(),
                        'port'              => $this->helper->getSmtpDetails()['port'],
                        'connection_class'  => $this->helper->getSmtpDetails()['auth'],
                        'connection_config' => $this->helper->getSmtpDetails(),
                    ]
                );

                if ($domain = $this->_scopeConfig->getValue('egoi/transactional/domain')) {
                    $message->getHeaders()->addHeaderLine('X-Domain', $domain);
                }

                $transport->setOptions($options);
                $transport->send($message);

            } else {
                $mail = new \Zend_Mail('utf-8');
                $mail->setBodyHtml(
                    'If you are receiving this message, everything seems to be ok with your SMTP configuration'
                );
                $mail->setFrom($salesEmail, $salesSender)
                     ->addTo($salesEmail, $salesSender)
                     ->setSubject('E-Goi / Magento - Test');

                if ($domain = $this->_scopeConfig->getValue('egoi/transactional/domain')) {
                    $mail->addHeader('X-Domain', $domain);
                }

                $mail->send($transport);

                $this->messageManager->addSuccessMessage(
                    'Success. Everything seems to be ok with your setting. We sent an email to ' . $salesEmail
                );
            }
        } catch (\Exception $e) {
            $this->messageManager->addErrorMessage('Error Testing your Settings: ' . $e->getMessage());
        }

        return $this->resultRedirectFactory->create()
                                           ->setUrl($this->_redirect->getRefererUrl());

    }
}
