<?php

/**
 * E-goi.com
 *
 * @title      E-Goi Multi-channel Marketing
 * @package    E-Goi
 * @copyright  Copyright (c) 2012-2023 E-Goi - https://e-goi.com
 */

namespace Egoi\Marketing\Controller\Qero;

/**
 *
 */
class Points extends \Magento\Framework\App\Action\Action
{

    /**
     * @var \Egoi\Marketing\Helper\Qero
     */
    protected $qeroHelper;

    /**
     * @param \Egoi\Marketing\Helper\Qero           $qeroHelper
     * @param \Magento\Framework\App\Action\Context $context
     */
    public function __construct(
        \Egoi\Marketing\Helper\Qero           $qeroHelper,
        \Magento\Framework\App\Action\Context $context
    )
    {

        $this->qeroHelper = $qeroHelper;

        parent::__construct($context);
    }

    /**
     *
     */
    public function execute()
    {

        /** @var \Magento\Backend\Model\View\Result\Redirect $resultRedirect */
        $resultRedirect = $this->resultRedirectFactory->create();

        if (!$this->qeroHelper->canUsePoints()) {

            return $resultRedirect->setRefererUrl();
        }

        $points = $this->getRequest()->getParam('qero_points');

        if ($points) {
            $points = preg_replace('/\D/', '', $this->getRequest()->getParam('qero_points'));
            $points = $points * 100;
        } else {
            $points = 0;
        }

        if ($points == 0 && $this->qeroHelper->getQueryAppliedPoints()) {
            $this->messageManager->addSuccessMessage(__("Points Removed"));
            $this->qeroHelper->getCustomerSession()->setData('qero_applied_points', 0);

            return $resultRedirect->setRefererUrl();

        }

        if ($points == 0) {
            return $resultRedirect->setRefererUrl();
        }

        if ($points > $this->qeroHelper->getQeroPoints()) {
            $this->messageManager->addErrorMessage(__("You don't have that many points available"));

            return $resultRedirect->setRefererUrl();

        }

        $balance = $this->qeroHelper->getQeroBalance();

        if (!is_array($balance)) {
            $this->messageManager->addErrorMessage(__("An error occurred. Please try again later"));

            return $resultRedirect->setRefererUrl();
        }

        $total = $this->qeroHelper->getCheckoutSession()->getQuote()->getBaseGrandTotal() * 100;
        $newValue = ceil($points * 100 / $total);

        if ($newValue > $balance['max_points_use_percentage']) {

            $maxPoints = $total * $balance['max_points_use_percentage'] / 100;
            $this->messageManager->addErrorMessage(__("You can only apply a maximum of %1 points", $maxPoints));

            return $resultRedirect->setRefererUrl();
        }

        $this->qeroHelper->getCustomerSession()->setData('qero_applied_points', $points);
        $this->messageManager->addSuccessMessage(__("Points Applied"));

        return $resultRedirect->setRefererUrl();

    }

}
