<?php

/**
 * E-goi.com
 *
 * @title      E-Goi Multi-channel Marketing
 * @package    E-Goi
 * @copyright  Copyright (c) 2012-2023 E-Goi - https://e-goi.com
 */

namespace Egoi\Marketing\Model;

/**
 * Class Autoresponders
 *
 * @package Egoi\Marketing\Model
 */
class Account extends \Magento\Framework\Model\AbstractModel
{

    /**
     * Prefix of model events names
     *
     * @var string
     */
    protected $_eventPrefix = 'egoi_account';

    /**
     * Parameter name in event
     *
     * In observe method you can use $observer->getEvent()->getObject() in this case
     *
     * @var string
     */
    protected $_eventObject = 'account';

    /**
     * @var Egoi
     */
    protected $egoi;

    /**
     * @var ResourceModel\Account\CollectionFactory
     */
    protected $accountCollection;

    /**
     * Account constructor.
     *
     * @param Egoi                                                         $egoi
     * @param ResourceModel\Account\CollectionFactory                      $egoiCollection
     * @param \Magento\Framework\Model\Context                             $context
     * @param \Magento\Framework\Registry                                  $registry
     * @param \Magento\Framework\Model\ResourceModel\AbstractResource|null $resource
     * @param \Magento\Framework\Data\Collection\AbstractDb|null           $resourceCollection
     * @param array                                                        $data
     */
    public function __construct(
        \Egoi\Marketing\Model\Egoi                                    $egoi,
        \Egoi\Marketing\Model\ResourceModel\Account\CollectionFactory $egoiCollection,
        \Magento\Framework\Model\Context                              $context,
        \Magento\Framework\Registry                                   $registry,
        \Magento\Framework\Model\ResourceModel\AbstractResource       $resource = null,
        \Magento\Framework\Data\Collection\AbstractDb                 $resourceCollection = null,
        array                                                         $data = []
    )
    {

        parent::__construct($context, $registry, $resource, $resourceCollection, $data);

        $this->egoi = $egoi;
        $this->accountCollection = $egoiCollection;
    }

    /**
     * Initialize resource model
     *
     * @return void
     */
    protected function _construct()
    {

        $this->_init('Egoi\Marketing\Model\ResourceModel\Account');
    }

    /**
     * @return $this|\Magento\Framework\DataObject
     */
    public function getAccount()
    {

        $col = $this->accountCollection->create()->getFirstItem();
        $tableName = $this->getResource()->getTable('egoi_account');
        if (!$col->getId()) {

            $d = $this->egoi->getAccountDetails();

            $db = $this->getResource()
                       ->getConnection()
                       ->describeTable($tableName);

            $final = array_intersect_key($d, $db);

            $this->getResource()->getConnection()->insert($tableName, $final);

            return $this->getCollection()->getFirstItem();
        }

        return $col;

    }

    function cron()
    {

        $account = $this->getAccount();
        $data = $this->egoi->getAccountDetails();

        if (!empty($data['account_id'])) {
            $account->setData($data)->save();
        }

    }
}
