<?php

/**
 * E-goi.com
 *
 * @title      E-Goi Multi-channel Marketing
 * @package    E-Goi
 * @copyright  Copyright (c) 2012-2023 E-Goi - https://e-goi.com
 */

namespace Egoi\Marketing\Model;

/**
 * Class Autoresponders
 *
 * @package Egoi\Marketing\Model
 */
class Lists extends \Magento\Framework\Model\AbstractModel
{

    /**
     * Prefix of model events names
     *
     * @var string
     */
    protected $_eventPrefix = 'egoi_lists';

    /**
     * Parameter name in event
     *
     * In observe method you can use $observer->getEvent()->getObject() in this case
     *
     * @var string
     */
    protected $_eventObject = 'lists';

    /**
     * @var Egoi
     */
    protected $egoi;

    /**
     * @var ExtraFactory
     */
    protected $_extraFactory;

    /**
     * @var ResourceModel\Extra\CollectionFactory
     */
    protected $_extraCollection;

    /**
     * @var ResourceModel\Lists\CollectionFactory
     */
    protected $_listsCollection;

    /**
     * @var \Magento\Store\Model\StoreManagerInterface
     */
    protected $_storeManager;

    /**
     * Initialize resource model
     *
     * @return void
     */
    protected function _construct()
    {

        $this->_init('Egoi\Marketing\Model\ResourceModel\Lists');
    }

    /**
     * Lists constructor.
     *
     * @param Egoi                                                         $egoi
     * @param \Magento\Store\Model\StoreManagerInterface                   $store
     * @param ExtraFactory                                                 $extraFactory
     * @param ResourceModel\Extra\CollectionFactory                        $extraCollection
     * @param ResourceModel\Lists\CollectionFactory                        $listsCollection
     * @param \Magento\Framework\Model\Context                             $context
     * @param \Magento\Framework\Registry                                  $registry
     * @param \Magento\Framework\Model\ResourceModel\AbstractResource|null $resource
     * @param \Magento\Framework\Data\Collection\AbstractDb|null           $resourceCollection
     * @param array                                                        $data
     */
    public function __construct(
        \Egoi\Marketing\Model\Egoi                                  $egoi,
        \Magento\Store\Model\StoreManagerInterface                  $store,
        \Egoi\Marketing\Model\ExtraFactory                          $extraFactory,
        \Egoi\Marketing\Model\ResourceModel\Extra\CollectionFactory $extraCollection,
        \Egoi\Marketing\Model\ResourceModel\Lists\CollectionFactory $listsCollection,
        \Magento\Framework\Model\Context                            $context,
        \Magento\Framework\Registry                                 $registry,
        \Magento\Framework\Model\ResourceModel\AbstractResource     $resource = null,
        \Magento\Framework\Data\Collection\AbstractDb               $resourceCollection = null,
        array                                                       $data = []
    )
    {

        parent::__construct($context, $registry, $resource, $resourceCollection, $data);

        $this->_storeManager = $store;
        $this->egoi = $egoi;
        $this->_extraFactory = $extraFactory;
        $this->_extraCollection = $extraCollection;
        $this->_listsCollection = $listsCollection;
    }

    /**
     * @return \Magento\Framework\Model\AbstractModel
     * @throws \Exception
     */
    public function beforeSave()
    {

        if ($this->getData('simple')) {
            return parent::beforeSave();
        }

        $model = $this->egoi;
        $data = $this->getData();

        if ($this->getId() &&
            (
                $this->getData('nome') != $this->getOrigData('nome') ||
                $this->getData('internal_name') != $this->getOrigData('internal_name')
            )
        ) {

            $model->updateList(
                $this->getData('listnum'),
                ['internal_name' => $this->getData('internal_name'), 'public_name' => $this->getData('nome')]
            );

        } else {

            $lists = $this->egoi->getLists($this->egoi->getListId());

            foreach ($lists as $list) {

                $list['extra_fields'] = $this->egoi->getExtraFields($list['list_id']);

                if (isset($list['extra_fields']) && is_array($list['extra_fields'])) {
                    foreach ($list['extra_fields'] as $field) {
                        if (isset($field['name']) && $field['name'] == 'store_id') {
                            $this->load($list['list_id'], 'listnum');
                            $this->setData('listnum', $list['list_id']);
                            break 2;
                        }
                    }
                }
            }

            $total = $this->_listsCollection->create()->getFirstItem();

            if ($total->getId()) {
                $this->setId($total->getId());
            }
        }

        return parent::beforeSave();

    }

    public function listExists()
    {

        $remoteList = $this->egoi->getLists();

        if (isset($remoteList['list_id'])) {
            $remoteList = [$remoteList];
        }

        $finalList = [];
        foreach ($remoteList as $index) {
            $finalList[$index['list_id']] = $index;
        }

        $model = $this->getCollection()->addFieldToFilter('listnum', ['in' => array_keys($finalList)])->getFirstItem();

        if ($model) {
            $model->setData('active', 1)->setData('simple', true)->save();
        }

        foreach ($this->getCollection() as $list) {
            if ($list->getId() != $model->getId()) {
                $list->delete();
            }
        }

        return $model;
    }

    /**
     * @return \Magento\Framework\DataObject
     */
    public function getList()
    {

        return $this->_listsCollection->create()->getFirstItem();
    }
}
