<?php

namespace Egoi\Marketing\Model\Total\Quote;

/**
 *
 */
class Egoi extends \Magento\Quote\Model\Quote\Address\Total\AbstractTotal
{

    /**
     * Discount calculation object
     *
     * @var \Magento\SalesRule\Model\Validator
     */
    protected $calculator;

    /**
     * Core event manager proxy
     *
     * @var \Magento\Framework\Event\ManagerInterface
     */
    protected $eventManager = null;

    /**
     * @var \Magento\Store\Model\StoreManagerInterface
     */
    protected $storeManager;

    /**
     * @var \Magento\Framework\Pricing\PriceCurrencyInterface
     */
    protected $priceCurrency;

    /**
     * @var \Egoi\Marketing\Helper\Data
     */
    protected $helper;

    /**
     * @param \Egoi\Marketing\Helper\Data                       $helper
     * @param \Magento\Framework\Event\ManagerInterface         $eventManager
     * @param \Magento\Store\Model\StoreManagerInterface        $storeManager
     * @param \Magento\SalesRule\Model\Validator                $validator
     * @param \Magento\Framework\Pricing\PriceCurrencyInterface $priceCurrency
     */
    public function __construct(
        \Egoi\Marketing\Helper\Data                       $helper,
        \Magento\Framework\Event\ManagerInterface         $eventManager,
        \Magento\Store\Model\StoreManagerInterface        $storeManager,
        \Magento\SalesRule\Model\Validator                $validator,
        \Magento\Framework\Pricing\PriceCurrencyInterface $priceCurrency
    )
    {

        $this->setCode('egoidiscount');
        $this->eventManager = $eventManager;
        $this->calculator = $validator;
        $this->storeManager = $storeManager;
        $this->priceCurrency = $priceCurrency;
        $this->helper = $helper;
    }

    public function collect(
        \Magento\Quote\Model\Quote                          $quote,
        \Magento\Quote\Api\Data\ShippingAssignmentInterface $shippingAssignment,
        \Magento\Quote\Model\Quote\Address\Total            $total
    )
    {

        $items = $shippingAssignment->getItems();
        if (!count($items)) {
            return $this;
        }
        parent::collect($quote, $shippingAssignment, $total);

        $customerSession = $this->helper->getCustomerSession();

        if ($quote->getBaseSubtotal() == 0 ||
            !$customerSession->getData('qero_applied_points')) {
            return $this;
        }

        $baseDiscountAmount = -($customerSession->getData('qero_applied_points') / 100);

        $discountAmount = $this->priceCurrency->convert($baseDiscountAmount);

        $quote->setData('qero_points', abs($customerSession->getData('qero_applied_points')));
        $label = 'Loyalty discount';

        $appliedCartDiscount = 0;
        if ($total->getDiscountDescription()) {
            $appliedCartDiscount = $total->getDiscountAmount();
            $discountAmount = $total->getDiscountAmount() + $discountAmount;
            $label = $total->getDiscountDescription() . ', ' . $label;
        }

        $total->setDiscountDescription($label);
        $total->setDiscountAmount($discountAmount);
        $total->setBaseDiscountAmount($baseDiscountAmount);
        $total->setSubtotalWithDiscount($total->getSubtotal() + $discountAmount);
        $total->setBaseSubtotalWithDiscount($total->getBaseSubtotal() + $baseDiscountAmount);

        if ($appliedCartDiscount) {
            $total->addTotalAmount($this->getCode(), $discountAmount - $appliedCartDiscount);
            $total->addBaseTotalAmount($this->getCode(), $baseDiscountAmount - $appliedCartDiscount);
        } else {
            $total->addTotalAmount($this->getCode(), $discountAmount);
            $total->addBaseTotalAmount($this->getCode(), $baseDiscountAmount);
        }

        $discountToItem = $discountAmount / $quote->getItemsQty();
        $baseDiscountToItem = $baseDiscountAmount / $quote->getItemsQty();
        foreach ($quote->getItems() as $item) {
            $item->setDiscountAmount($item->getDiscountAmount() + abs($discountToItem));
            $item->setBaseDiscountAmount($item->getBaseDiscountAmount() + abs($baseDiscountToItem));
        }

        return $this;
    }

    /**
     * Add discount total information to address
     *
     * @param \Magento\Quote\Model\Quote               $quote
     * @param \Magento\Quote\Model\Quote\Address\Total $total
     *
     * @return array|null
     */
    public function fetch(\Magento\Quote\Model\Quote $quote, \Magento\Quote\Model\Quote\Address\Total $total)
    {

        $result = null;
        $amount = $total->getDiscountAmount();

        if ($amount != 0) {
            $description = $total->getDiscountDescription();
            $result = [
                'code'  => $this->getCode(),
                'title' => !empty($description) ? __('Discount (%1)', $description) : __('Discount'),
                'value' => $amount,
            ];
        }

        return $result;

    }
}
