<?php

namespace Egoi\Marketing\Observer;

use Magento\Framework\Event\ObserverInterface;

/**
 * Class NewOrder
 *
 * @package Egoi\Marketing\Observer
 */
class NewOrder implements ObserverInterface
{

    /**
     * @var \Egoi\Marketing\Model\ConversionsFactory
     */
    protected $conversionsFactory;

    /**
     * @var \Egoi\Marketing\Model\AutorespondersFactory
     */
    protected $autorespondersFactory;

    /**
     * @var \Magento\Framework\App\Config\ScopeConfigInterface
     */
    protected $scopeConfig;

    /**
     * @var \Magento\Newsletter\Model\SubscriptionManagerFactory
     */
    protected $subscriptionManagerFactory;

    /**
     * @var \Egoi\Marketing\Helper\Data
     */
    protected $egoiHelper;

    /**
     * @param \Egoi\Marketing\Model\ConversionsFactory             $conversionsFactory
     * @param \Egoi\Marketing\Helper\Data                          $egoiHelper
     * @param \Magento\Framework\App\Config\ScopeConfigInterface   $scopeConfig
     * @param \Magento\Newsletter\Model\SubscriptionManagerFactory $subscriptionManagerFactory
     * @param \Egoi\Marketing\Model\AutorespondersFactory          $autorespondersFactory
     */
    function __construct(
        \Egoi\Marketing\Model\ConversionsFactory             $conversionsFactory,
        \Egoi\Marketing\Helper\Data                          $egoiHelper,
        \Magento\Framework\App\Config\ScopeConfigInterface   $scopeConfig,
        \Magento\Newsletter\Model\SubscriptionManagerFactory $subscriptionManagerFactory,
        \Egoi\Marketing\Model\AutorespondersFactory          $autorespondersFactory
    )
    {

        $this->conversionsFactory = $conversionsFactory;
        $this->egoiHelper = $egoiHelper;
        $this->scopeConfig = $scopeConfig;
        $this->subscriptionManagerFactory = $subscriptionManagerFactory;
        $this->autorespondersFactory = $autorespondersFactory;
    }

    /**
     * @param \Magento\Framework\Event\Observer $event
     */
    public function execute(\Magento\Framework\Event\Observer $event)
    {

        try {
            $this->autorespondersFactory->create()->newOrder($event);
            $this->conversionsFactory->create()->newConversion($event);

            /** @var \Magento\Sales\Model\Order $order */
            $order = $event->getEvent()->getOrder();

            if ($this->scopeConfig->getValue('egoi/info/auto')) {
                $email = $order->getCustomerEmail();
                $this->subscriptionManagerFactory->create()
                                                 ->subscribe(
                                                     $email,
                                                     $this->egoiHelper->getStoreManagerInterface()
                                                                      ->getStore()
                                                                      ->getId()
                                                 );
            }

        } catch (\Exception $e) {

        }
    }

}
