<?php

namespace Egoi\Marketing\Plugin;

/**
 * Class Subscriber
 *
 * @package Egoi\Marketing\Plugin
 */
class Subscriber
{

    /**
     * @var \Magento\Framework\App\Config\ScopeConfigInterface
     */
    protected $scopeConfig;

    /**
     * @var \Egoi\Marketing\Model\SubscriberFactory
     */
    protected $subscriberFactory;

    /**
     * @var \Egoi\Marketing\Helper\Data
     */
    protected $helper;

    /**
     * @param \Egoi\Marketing\Helper\Data                        $helper
     * @param \Egoi\Marketing\Model\SubscriberFactory            $subscriberFactory
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
     */
    public function __construct(
        \Egoi\Marketing\Helper\Data                        $helper,
        \Egoi\Marketing\Model\SubscriberFactory            $subscriberFactory,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
    )
    {

        $this->helper = $helper;
        $this->scopeConfig = $scopeConfig;
        $this->subscriberFactory = $subscriberFactory;
    }

    /**
     * @param \Magento\Newsletter\Model\Subscriber $subject
     * @param                                      $result
     *
     * @return bool
     */
    public function afterIsSubscribed(\Magento\Newsletter\Model\Subscriber $subject, $result)
    {

        if ($subject->getInCallBack()) {
            return $result;
        }

        if (!filter_var($subject->getEmail(), FILTER_VALIDATE_EMAIL)) {
            return $result;
        }

        return !empty($this->helper->getSubscriberCookie());
        //
        //        $remote = $this->subscriberFactory->create()->isSubscribedByEmail($subject->getEmail());
        //
        //        if (!$remote && $result) {
        //            $subject->loadByEmail($subject->getEmail())->setImportMode(true)->unsubscribe();
        //        }
        //
        //        return $remote;

    }

    /**
     * @param \Magento\Newsletter\Model\Subscriber $subject
     * @param                                      $result
     *
     * @return mixed
     */
    public function afterSubscribe(\Magento\Newsletter\Model\Subscriber $subject, $result)
    {

        if ($subject->getInCallBack()) {
            return $result;
        }

        if (!filter_var($subject->getEmail(), FILTER_VALIDATE_EMAIL)) {
            return $result;
        }

        try {
            $this->subscriberFactory->create()
                                    ->subscribe($subject->getEmail(), $subject->getStoreId());

        } catch (\Exception $e) {

        }

        return $result;
    }

    /**
     * @param \Magento\Newsletter\Model\Subscriber $subject
     * @param                                      $result
     *
     * @return mixed
     */
    public function afterSubscribeCustomerById(\Magento\Newsletter\Model\Subscriber $subject, $result)
    {

        if ($subject->getInCallBack() ||
            !isset($_POST['is_subscribed']) ||
            $_POST['is_subscribed'] == 0) {

            return $result;
        }

        if (!filter_var($subject->getEmail(), FILTER_VALIDATE_EMAIL)) {
            return $result;
        }

        try {
            $this->subscriberFactory->create()
                                    ->subscribe($subject->getEmail(), $subject->getStoreId());

        } catch (\Exception $e) {

        }

        return $result;
    }

    /**
     * @param \Magento\Newsletter\Model\Subscriber $subject
     * @param                                      $result
     *
     * @return mixed
     */
    public function afterUnsubscribe(\Magento\Newsletter\Model\Subscriber $subject, $result)
    {

        if ($subject->getInCallBack()) {
            return $result;
        }

        if (!filter_var($subject->getEmail(), FILTER_VALIDATE_EMAIL)) {
            return $result;
        }

        try {
            $this->subscriberFactory->create()->unsubscribe($subject->getEmail());
        } catch (\Exception $e) {

        }

        return $result;
    }

    /**
     * @param \Magento\Newsletter\Model\Subscriber $subject
     * @param                                      $result
     *
     * @return mixed
     */
    public function afterUnsubscribeCustomerById(\Magento\Newsletter\Model\Subscriber $subject, $result)
    {

        if ($subject->getInCallBack()) {
            return $result;
        }

        if (!filter_var($subject->getEmail(), FILTER_VALIDATE_EMAIL)) {
            return $result;
        }

        if (isset($_POST['is_subscribed']) && $_POST['is_subscribed'] == 1) {
            return $result;
        }

        try {
            $this->subscriberFactory->create()->unsubscribe($subject->getEmail());
        } catch (\Exception $e) {

        }

        return $result;
    }
}
